from PyQt5 import QtWidgets, QtCore
from typing import List

import todo.gui.tasks.dialog
from todo.model.status import Status
from todo.model.task import Task, ValidTaskForm
from todo.model.tag import Tag

def open(table: QtWidgets.QTableWidget, status: Status, update_task_signal, position):
    rows = set([index.row() for index in table.selectedIndexes()])

    menu = QtWidgets.QMenu(table)

    if len(rows) == 1:
        modify_action = menu.addAction(todo.gui.icon.dialog_open(menu.style()), "Modifier")
    else:
        modify_action = QtWidgets.QAction(menu)

    delete_action = menu.addAction(todo.gui.icon.trash(menu.style()), "Supprimer")

    if status != Status.READY:
        move_to_ready = menu.addAction(todo.gui.icon.task_ready(menu.style()), "Mettre à prêt")
    else:
        move_to_ready = QtWidgets.QAction(menu)

    if status != Status.WAITING:
        move_to_waiting = menu.addAction(todo.gui.icon.task_waiting(menu.style()), "Mettre en attente")
    else:
        move_to_waiting = QtWidgets.QAction(menu)

    if status != Status.MAYBE:
        move_to_maybe = menu.addAction(todo.gui.icon.task_maybe(menu.style()), "Mettre à peut-être")
    else:
        move_to_maybe = QtWidgets.QAction(menu)

    action = menu.exec_(table.mapToGlobal(position + QtCore.QPoint(15, 20)))
    if action == modify_action and len(rows) == 1:
        row = list(rows)[0]
        (task, tags) = table.get_at(row)
        todo.gui.tasks.dialog.update(table, update_task_signal, row, task, tags).exec_()
    elif action == delete_action and len(rows) > 0:
        todo.gui.tasks.dialog.confirm_delete(table, rows, lambda: table.delete_rows(rows))
    elif action == move_to_ready:
        table.update_status(rows, Status.READY)
    elif action == move_to_waiting:
        table.update_status(rows, Status.WAITING)
    elif action == move_to_maybe:
        table.update_status(rows, Status.MAYBE)
